//////////////////////
#include <OneWire.h>
#include <DallasTemperature.h>
// 1-wire Dallas Data wire is plugged into port 2 on the Arduino
#define ONE_WIRE_BUS 2
// Setup a oneWire instance to communicate with any OneWire devices (not just Maxim/Dallas temperature ICs)
OneWire oneWire(ONE_WIRE_BUS);
// Pass our oneWire reference to Dallas Temperature. 
DallasTemperature sensors(&oneWire);
// arrays to hold device address
DeviceAddress DallasThermometer;
///////////////
#include <SPI.h>
#include <Ethernet.h>

//IPAddress ip(192,168,0,177); // ip in lan
//IPAddress gateway(192,168,0,1); // internet access via router
//IPAddress subnet(255,255,255,0); //subnet mask
//IPAddress NASserver(192,168,0,101); 

char NASserver[] = "your_NAS_domain_name";
EthernetServer server(80); //Arduino server port  
EthernetClient client;
///////
#include <dht.h> //for DHT22, but suporting DHT11
dht DHT;
#define DHT22_PIN1 6
#define DHT22_PIN2 7
///////////
//GY-65 (BMP05) sensor
//connection: 
// Vcc: to +3.3V (only!, not 5V!)
// GND-:to GND
// SDA: pin A4 (i2c data)
// SCL: pin A5 (i2c clock)
// other pins are not using
#include <Wire.h>
#include <BMP085.h>
BMP085 bmp;
// ------------------------------------ configuration to edit is here -----------------------------------------------------------
//#include <Arduino_1.h>
//#include <Arduino_2.h>
byte mac[] = { 0xDE, 0xAD, 0xBE, 0xEF, 0xAA, 0x01 }; //physical mac address
//byte mac[] = { 0xDE, 0xAD, 0xBE, 0xEF, 0xBB, 0x02 }; //physical mac address
byte SID_arr[] = {1, 2, 3, 4, 5, 6, 7, 8}; //array of Sensors ID
//byte SID_arr[] = {11, 12, 13, 14, 15, 16, 17, 18}; //array of Sensors ID
char* names[]={"Humidity (%RH, DHT11#1)", 
               "Temperature (C, DHT11#1)",

               "Temperature (C, BMP05#1)", 
               "Pressure (mmHg, BMP05#1)",

               "Humidity (%RH, DHT22#1)", 
               "Temperature (C, DHT22#1)",

               "Humidity (%RH, DHT22#2)", 
               "Temperature (C, DHT22#2)"};
               
// -----------------------------------------------------------------------------------------------
//not enough memory on UNO for LCD
//#include <LiquidCrystal_I2C.h> //LCD I2C
//LiquidCrystal_I2C lcd(0x27, 2, 1, 0, 4, 5, 6, 7, 3, POSITIVE); // LCD I2C: Addr, En, Rw, Rs, d4, d5, d6, d7, backlighpin, polarity

unsigned long LastMeasMS = 0; //mS
const unsigned long MeasInterval = 2000; //mS
//unsigned long AveCNT=1; //average counter;
//unsigned long MeasCNT = 1; //measurement counter
const unsigned long AverageFactor = 150; //times to average, every 5 minutes - 150

const int valuesNum = 8; //"values" and "names" array length
//double values [valuesNum]; //values
//double valuesSum [valuesNum]; //values

#include "StatisticLastValue.h"
//Statistic myStats; 
Statistic stat[valuesNum];

    
// function to print a Dallas device address
//void printAddress(DeviceAddress deviceAddress)
void printAddress(DeviceAddress deviceAddress)
{
  for (uint8_t i = 0; i < 8; i++)
  {
    if (deviceAddress[i] < 16) Serial.print("0");
    Serial.print(deviceAddress[i], HEX);
  }
}
                        
void setup()
{
  delay(2000);// give the ethernet module time to boot up
  Serial.begin(9600); 
  Ethernet.begin(mac); //use DHCP. Was: Ethernet.begin(mac, ip, subnet, gateway); 
  server.begin();
 // Serial.println("Arduino sensors server/client"); // keep track of what is loaded
  Serial.print("IP address: "); Serial.println(Ethernet.localIP());
  /*
  lcd.begin(20,4);               // initialize the lcd 
  lcd.home ();                   // go home
  lcd.setCursor(0,  0); lcd.print("IP: "); lcd.print(Ethernet.localIP());
  delay(1000);
  lcd.clear();
  */
  
  LastMeasMS = 0; //millis();  
 // pinMode(10, OUTPUT); //sd card
  bmp.begin();//bmp085 sensor
  
  //dallas temp. sensor (DS18***)
  sensors.begin();
  //Serial.print("Found "); Serial.print(sensors.getDeviceCount(), DEC);  Serial.println(" Dallas 1-wire device(s).");
  if (!sensors.getAddress(DallasThermometer, 0)) ;//Serial.println("Unable to find address for Dev0");
  //Serial.print("Dev0 Addr: ");
  //printAddress(DallasThermometer);
  //Serial.println();
  sensors.setResolution(DallasThermometer, 11);
//clear stat  
  for (int i=0; i<valuesNum; i++)
    {stat[i].clear();}
}

void loop()
{
  //////////////////////////////////////////////////////////////////////////////
  unsigned long currentMillis = millis();
  if (currentMillis-LastMeasMS > MeasInterval) 
    {//time to do measurement 
    LastMeasMS=currentMillis;

    //time to perform measurement
    getMeas();

    //ShowLCD(values); //lack of memory on UNO
    
    //print measured values
    Serial.println(stat[0].count());
    for (int i=0; i<valuesNum; i++)
        {
        Serial.print(stat[i].last());
        if (i<(valuesNum-1)) {Serial.print(", ");} else {Serial.println();}
        }   

    //stat
    /*
    for (int i=0; i<valuesNum; i++)
      {
      Serial.print("Sensor#: ");
      Serial.print(i+1);
      Serial.print(", CNT: ");
      Serial.print(stat[i].count());
      Serial.print(", Ave: ");
      Serial.println(stat[i].average());
      }            
    */
    
  } //time to do measurement    
  //////////////////////////////////////////////////////////////////////////////
if(!client.connected() && (stat[0].count() >= AverageFactor)) //(millis() - lastConnectionTime > postingInterval)) 
  {
  sendREQ(); //call sendREQ function  
  for (int i=0; i<valuesNum; i++)  {stat[i].clear();}
  }  

//////////////////////////////////////////////////////
  EthernetClient Ard_client = server.available();
  if (Ard_client) {
    while (Ard_client.connected()) {
      if (Ard_client.available()) {
        char c = Ard_client.read();
        //if HTTP request has ended
        if (c == '\n') {
            //now output HTML data header
          Ard_client.println("HTTP/1.1 200 OK");
          Ard_client.println("Content-Type: text/html");
          Ard_client.println();
          Ard_client.println("<!DOCTYPE HTML>");
          Ard_client.println("<html>");

          Ard_client.println("<HEAD>");
          Ard_client.println("<TITLE>Arduino Sensors page</TITLE>");
          Ard_client.println("</HEAD>");
          
                    // add a meta refresh tag, so the browser pulls again every nn seconds:
          Ard_client.println("<meta http-equiv=\"refresh\" content=\"10\">");
          //-------------
          Ard_client.println("<H1> Arduino Sensors page </H1>");
          Ard_client.println("AverageFactor: "); Ard_client.println(AverageFactor); Ard_client.println("<br>");
          //table
          Ard_client.print("<table border='1'>");
          Ard_client.print("<tr  bgcolor='#66FF66'>"); //c0c0c0
          Ard_client.print("<th>Sensor</th>");
          Ard_client.print("<th>SID</th>");
          Ard_client.print("<th>Count</th>");
          Ard_client.print("<th>AVE</th>");
          Ard_client.print("<th>MIN</th>");
          Ard_client.print("<th>MAX</th>");
          Ard_client.print("<th>STD_DEV</th>");
          Ard_client.print("</tr>");
          for (int i=0; i<valuesNum; i++) 
           {
             Ard_client.print("<tr>");
             //
             Ard_client.print("<td>"); Ard_client.print(names[i]);            Ard_client.print("</td>");

             Ard_client.print("<td>"); Ard_client.print(SID_arr[i]);            Ard_client.print("</td>");
             
             Ard_client.print("<td>"); Ard_client.print(stat[i].count());     Ard_client.print("</td>");             
             Ard_client.print("<td>"); Ard_client.print(stat[i].average());   Ard_client.print("</td>");
             Ard_client.print("<td>"); Ard_client.print(stat[i].minimum());   Ard_client.print("</td>");
             Ard_client.print("<td>"); Ard_client.print(stat[i].maximum());   Ard_client.print("</td>");             
             Ard_client.print("<td>"); Ard_client.print(stat[i].pop_stdev()); Ard_client.print("</td>");                          
             //
             Ard_client.print("</tr>");
           }
          Ard_client.print("</table>");
          Ard_client.println("</html>");
          delay(1);
          //stopping client
          Ard_client.stop();
        }
      }
    }
  }
} 

//////////////////////////
void sendREQ() //client function to send/receie GET request data.
{
  if (client.connect(NASserver, 80)) 
  {
  Serial.println("connected");
  client.print("GET /Arduino/SaveArduino2.php?");
  for (int i=0; i<valuesNum; i++)
    {
    client.print("&SID");
    client.print(SID_arr[i]);

//    client.print("AVE=");
     client.print("=");

    client.print(stat[i].average());   client.print(",");
    client.print(stat[i].minimum());   client.print(",");
    client.print(stat[i].maximum());   client.print(",");
    client.print(stat[i].pop_stdev()); //client.print(",");
    
    
    }
  client.println("&Password=some_password7295 HTTP/1.0")
  client.println();
  } 
  else 
  {
    Serial.println("connection failed");
    Serial.println();
  }

  while(client.connected() && !client.available()) delay(1); //waits for data
  while (client.connected() || client.available()) 
  { //connected or data available
    char c = client.read();
    Serial.print(c);
  }
  Serial.println();
  Serial.println("disconnecting.");
  Serial.println("==================");
  Serial.println();
  client.stop();
}

/*
void ShowLCD(double aValues[])
{
lcd.setCursor(0,  0); lcd.print("S1:"); lcd.print(aValues[0], 0); lcd.print("?");
lcd.setCursor(10, 0); lcd.print("S2:"); lcd.print(aValues[1], 0); lcd.print("C");
lcd.setCursor(0,  1); lcd.print("S3:"); lcd.print(aValues[2], 0); lcd.print("C");
lcd.setCursor(10, 1); lcd.print("S4:"); lcd.print(aValues[3], 0); lcd.print("mmHg");
lcd.setCursor(0,  2); lcd.print("S5:"); lcd.print(aValues[4], 0); lcd.print("%RH");
lcd.setCursor(10, 2); lcd.print("S6:"); lcd.print(aValues[5], 0); lcd.print("C");
lcd.setCursor(0,  3); lcd.print("S7:"); lcd.print(aValues[6], 0); lcd.print("%RH");
lcd.setCursor(10, 3); lcd.print("S8:"); lcd.print(aValues[7], 0); lcd.print("C");
}
*/

void getMeas() //fillout aValues with measurements
{
//1 Pascal = 0.00750061683 Torr (mmHg)
//1 Torr (mmHg) = 133.322368 Pascals
//int chkDHT11 = DHT11.read(DHT11PIN);
int err=-999;
int chk;
double value;
  
//Dallas temp.
///////////////////////////////
sensors.requestTemperatures(); // Send the command to get temperatures
value = sensors.getTempC(DallasThermometer);
stat[1].add(value);

stat[0].add(err); //no sensor

//DHT22#1
chk = DHT.read22(DHT22_PIN1);
if (chk==DHTLIB_OK)
  {
   value = DHT.humidity;// %RH
   stat[4].add(value);
   value =  DHT.temperature; //C
   stat[5].add(value);
  }
  else 
  {
   stat[4].add(err);
   stat[5].add(err); 
  }

//DHT22#2
chk = DHT.read22(DHT22_PIN2);
if (chk==DHTLIB_OK)
  {
    value =  DHT.humidity;// %RH
   stat[6].add(value);
   value = DHT.temperature; //C
   stat[7].add(value); 
  }
  else 
  {
   stat[6].add(err);
   stat[7].add(err); 
  }

value = bmp.readTemperature(); //C
stat[2].add(value);
value = bmp.readPressure() * 0.00750061683; //from Pa to mmHg
stat[3].add(value); 
}

